# Stubs for galaxy.tools.parser.interface (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

NOT_IMPLEMENTED_MESSAGE = ...  # type: str

class ToolSource:
    default_is_multi_byte = ...  # type: bool
    def parse_id(self): ...
    def parse_version(self): ...
    def parse_tool_module(self): ...
    def parse_action_module(self): ...
    def parse_tool_type(self): ...
    def parse_name(self): ...
    def parse_description(self): ...
    def parse_is_multi_byte(self): ...
    def parse_display_interface(self, default): ...
    def parse_require_login(self, default): ...
    def parse_request_param_translation_elem(self): ...
    def parse_command(self): ...
    def parse_environment_variables(self): ...
    def parse_interpreter(self): ...
    def parse_redirect_url_params_elem(self): ...
    def parse_version_command(self): ...
    def parse_version_command_interpreter(self): ...
    def parse_parallelism(self): ...
    def parse_hidden(self): ...
    def parse_sanitize(self): ...
    def parse_refresh(self): ...
    def parse_requirements_and_containers(self): ...
    def parse_input_pages(self): ...
    def parse_outputs(self, tool): ...
    def parse_strict_shell(self): ...
    def parse_stdio(self): ...
    def parse_help(self): ...
    def parse_profile(self): ...
    def parse_tests_to_dict(self): ...

class PagesSource:
    page_sources = ...  # type: Any
    def __init__(self, page_sources) -> None: ...
    @property
    def inputs_defined(self): ...

class PageSource:
    def parse_display(self): ...
    def parse_input_sources(self): ...

class InputSource:
    default_optional = ...  # type: bool
    def elem(self): ...
    def get(self, key, value: Optional[Any] = ...): ...
    def get_bool(self, key, default): ...
    def parse_label(self): ...
    def parse_help(self): ...
    def parse_sanitizer_elem(self): ...
    def parse_validator_elems(self): ...
    def parse_optional(self, default: Optional[Any] = ...): ...
    def parse_dynamic_options_elem(self): ...
    def parse_static_options(self): ...
    def parse_conversion_tuples(self): ...
    def parse_nested_inputs_source(self): ...
    def parse_test_input_source(self): ...
    def parse_when_input_sources(self): ...

class ToolStdioRegex:
    match = ...  # type: str
    stdout_match = ...  # type: bool
    stderr_match = ...  # type: bool
    error_level = ...  # type: str
    desc = ...  # type: str
    def __init__(self) -> None: ...

class ToolStdioExitCode:
    range_start = ...  # type: Any
    range_end = ...  # type: Any
    error_level = ...  # type: str
    desc = ...  # type: str
    def __init__(self) -> None: ...

class TestCollectionDef:
    elements = ...  # type: Any
    collection_type = ...  # type: Any
    name = ...  # type: Any
    def __init__(self, elem, parse_param_elem) -> None: ...
    def collect_inputs(self): ...

class TestCollectionOutputDef:
    name = ...  # type: Any
    collection_type = ...  # type: Any
    count = ...  # type: Any
    attrib = ...  # type: Any
    element_tests = ...  # type: Any
    def __init__(self, name, attrib, element_tests) -> None: ...



# Stubs for galaxy.tools.locations.file (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from ..locations import ToolLocationResolver as ToolLocationResolver

class HttpToolResolver(ToolLocationResolver):
    scheme = ...  # type: str
    def get_tool_source_path(self, uri_like): ...



# Stubs for galaxy.tools.cwl.runtime_actions (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def handle_outputs(job_directory: Optional[Any] = ...): ...



# Stubs for galaxy.util.inflection (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Base:
    def cond_plural(self, number_of_records, word): ...
    def titleize(self, word, uppercase: str = ...): ...
    def camelize(self, word): ...
    def underscore(self, word): ...
    def humanize(self, word, uppercase: str = ...): ...
    def variablize(self, word): ...
    def tableize(self, class_name): ...
    def classify(self, table_name): ...
    def ordinalize(self, number): ...
    def unaccent(self, text): ...
    def string_replace(self, word, find, replace): ...
    def urlize(self, text): ...
    def demodulize(self, module_name): ...
    def modulize(self, module_description): ...
    def foreignKey(self, class_name, separate_class_name_and_id_with_underscore: int = ...): ...

class English(Base):
    def pluralize(self, word): ...
    def singularize(self, word): ...

class Inflector:
    Inflector = ...  # type: Any
    def __init__(self, Inflector: Any = ...) -> None: ...
    def pluralize(self, word): ...
    def singularize(self, word): ...
    def cond_plural(self, number_of_records, word): ...
    def titleize(self, word, uppercase: str = ...): ...
    def camelize(self, word): ...
    def underscore(self, word): ...
    def humanize(self, word, uppercase: str = ...): ...
    def variablize(self, word): ...
    def tableize(self, class_name): ...
    def classify(self, table_name): ...
    def ordinalize(self, number): ...
    def unaccent(self, text): ...
    def urlize(self, text): ...
    def demodulize(self, module_name): ...
    def modulize(self, module_description): ...
    def foreignKey(self, class_name, separate_class_name_and_id_with_underscore: int = ...): ...



# Stubs for galaxy.tools.parser.yaml (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from .interface import InputSource as InputSource
from .interface import PageSource as PageSource
from .interface import PagesSource as PagesSource
from .interface import ToolSource as ToolSource
from .output_actions import ToolOutputActionGroup as ToolOutputActionGroup
from .output_collection_def import dataset_collector_descriptions_from_list as dataset_collector_descriptions_from_list
from .output_objects import ToolOutput as ToolOutput, ToolOutputCollection as ToolOutputCollection, ToolOutputCollectionStructure as ToolOutputCollectionStructure
from .util import error_on_exit_code as error_on_exit_code

class YamlToolSource(ToolSource):
    root_dict = ...  # type: Any
    def __init__(self, root_dict, source_path: Optional[Any] = ...) -> None: ...
    def parse_id(self): ...
    def parse_version(self): ...
    def parse_name(self): ...
    def parse_description(self): ...
    def parse_edam_operations(self): ...
    def parse_edam_topics(self): ...
    def parse_is_multi_byte(self): ...
    def parse_sanitize(self): ...
    def parse_display_interface(self, default): ...
    def parse_require_login(self, default): ...
    def parse_command(self): ...
    def parse_environment_variables(self): ...
    def parse_interpreter(self): ...
    def parse_version_command(self): ...
    def parse_version_command_interpreter(self): ...
    def parse_requirements_and_containers(self): ...
    def parse_input_pages(self): ...
    def parse_strict_shell(self): ...
    def parse_stdio(self): ...
    def parse_help(self): ...
    def parse_outputs(self, tool): ...
    def parse_tests_to_dict(self): ...
    def parse_profile(self): ...

class YamlPageSource(PageSource):
    inputs_list = ...  # type: Any
    def __init__(self, inputs_list) -> None: ...
    def parse_input_sources(self): ...

class YamlInputSource(InputSource):
    input_dict = ...  # type: Any
    def __init__(self, input_dict) -> None: ...
    def get(self, key, default: Optional[Any] = ...): ...
    def get_bool(self, key, default): ...
    def parse_input_type(self): ...
    def parse_nested_inputs_source(self): ...
    def parse_test_input_source(self): ...
    def parse_when_input_sources(self): ...
    def parse_static_options(self): ...



# Stubs for galaxy.tools.deps.mulled.mulled_build (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
import installable
from sys import platform as _platform

class BuildExistsException(Exception): ...

class InvolucroContext(installable.InstallableContext):
    installable_description = ...  # type: str
    involucro_bin = ...  # type: str
    shell_exec = ...  # type: Any
    verbose = ...  # type: Any
    def __init__(self, involucro_bin: Optional[Any] = ..., shell_exec: Optional[Any] = ..., verbose: str = ...) -> None: ...
    def build_command(self, involucro_args): ...
    def exec_command(self, involucro_args): ...
    def is_installed(self): ...
    def can_install(self): ...
    @property
    def parent_path(self): ...

def main(argv: Optional[Any] = ...): ...



# Stubs for networkx.algorithms.boundary (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def edge_boundary(G, nbunch1, nbunch2: Optional[Any] = ..., data: bool = ..., keys: bool = ..., default: Optional[Any] = ...): ...
def node_boundary(G, nbunch1, nbunch2: Optional[Any] = ...): ...



# Stubs for networkx.algorithms.shortest_paths.weighted (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def dijkstra_path(G, source, target, weight: str = ...): ...
def dijkstra_path_length(G, source, target, weight: str = ...): ...
def single_source_dijkstra_path(G, source, cutoff: Optional[Any] = ..., weight: str = ...): ...
def single_source_dijkstra_path_length(G, source, cutoff: Optional[Any] = ..., weight: str = ...): ...
def single_source_dijkstra(G, source, target: Optional[Any] = ..., cutoff: Optional[Any] = ..., weight: str = ...): ...
def multi_source_dijkstra_path(G, sources, cutoff: Optional[Any] = ..., weight: str = ...): ...
def multi_source_dijkstra_path_length(G, sources, cutoff: Optional[Any] = ..., weight: str = ...): ...
def multi_source_dijkstra(G, sources, target: Optional[Any] = ..., cutoff: Optional[Any] = ..., weight: str = ...): ...
def dijkstra_predecessor_and_distance(G, source, cutoff: Optional[Any] = ..., weight: str = ...): ...
def all_pairs_dijkstra(G, cutoff: Optional[Any] = ..., weight: str = ...): ...
def all_pairs_dijkstra_path_length(G, cutoff: Optional[Any] = ..., weight: str = ...): ...
def all_pairs_dijkstra_path(G, cutoff: Optional[Any] = ..., weight: str = ...): ...
def bellman_ford_predecessor_and_distance(G, source, target: Optional[Any] = ..., cutoff: Optional[Any] = ..., weight: str = ...): ...
def bellman_ford_path(G, source, target, weight: str = ...): ...
def bellman_ford_path_length(G, source, target, weight: str = ...): ...
def single_source_bellman_ford_path(G, source, cutoff: Optional[Any] = ..., weight: str = ...): ...
def single_source_bellman_ford_path_length(G, source, cutoff: Optional[Any] = ..., weight: str = ...): ...
def single_source_bellman_ford(G, source, target: Optional[Any] = ..., cutoff: Optional[Any] = ..., weight: str = ...): ...
def all_pairs_bellman_ford_path_length(G, cutoff: Optional[Any] = ..., weight: str = ...): ...
def all_pairs_bellman_ford_path(G, cutoff: Optional[Any] = ..., weight: str = ...): ...
def goldberg_radzik(G, source, weight: str = ...): ...
def negative_edge_cycle(G, weight: str = ...): ...
def bidirectional_dijkstra(G, source, target, weight: str = ...): ...
def johnson(G, weight: str = ...): ...



# Stubs for galaxy.tools.linters.cwl (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

lint_tool_types = ...  # type: Any

def lint_cwl_validation(tool_source, lint_ctx): ...
def lint_new_draft(tool_source, lint_ctx): ...
def lint_docker_image(tool_source, lint_ctx): ...
def lint_description(tool_source, lint_ctx): ...



# Stubs for galaxy.tools.deps.singularity_util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def build_singularity_run_command(container_command, image, volumes: Any = ..., env: Any = ..., working_directory: Any = ..., singularity_cmd: Any = ..., run_extra_arguments: Any = ..., sudo: Any = ..., sudo_cmd: Any = ...): ...



# Stubs for galaxy.tools.linters.stdio (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .command import get_command as get_command

def lint_stdio(tool_source, lint_ctx): ...



# Stubs for networkx.drawing.layout (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def random_layout(G, center: Optional[Any] = ..., dim: int = ..., random_state: Optional[Any] = ...): ...
def circular_layout(G, scale: int = ..., center: Optional[Any] = ..., dim: int = ...): ...
def shell_layout(G, nlist: Optional[Any] = ..., scale: int = ..., center: Optional[Any] = ..., dim: int = ...): ...
def fruchterman_reingold_layout(G, k: Optional[Any] = ..., pos: Optional[Any] = ..., fixed: Optional[Any] = ..., iterations: int = ..., threshold: float = ..., weight: str = ..., scale: int = ..., center: Optional[Any] = ..., dim: int = ..., random_state: Optional[Any] = ...): ...
spring_layout = fruchterman_reingold_layout

def kamada_kawai_layout(G, dist: Optional[Any] = ..., pos: Optional[Any] = ..., weight: str = ..., scale: int = ..., center: Optional[Any] = ..., dim: int = ...): ...
def spectral_layout(G, weight: str = ..., scale: int = ..., center: Optional[Any] = ..., dim: int = ...): ...
def rescale_layout(pos, scale: int = ...): ...



# Stubs for networkx.convert_matrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def to_pandas_adjacency(G, nodelist: Optional[Any] = ..., dtype: Optional[Any] = ..., order: Optional[Any] = ..., multigraph_weight: Any = ..., weight: str = ..., nonedge: float = ...): ...
def from_pandas_adjacency(df, create_using: Optional[Any] = ...): ...
def to_pandas_edgelist(G, source: str = ..., target: str = ..., nodelist: Optional[Any] = ..., dtype: Optional[Any] = ..., order: Optional[Any] = ...): ...
def from_pandas_edgelist(df, source: str = ..., target: str = ..., edge_attr: Optional[Any] = ..., create_using: Optional[Any] = ...): ...
def to_numpy_matrix(G, nodelist: Optional[Any] = ..., dtype: Optional[Any] = ..., order: Optional[Any] = ..., multigraph_weight: Any = ..., weight: str = ..., nonedge: float = ...): ...
def from_numpy_matrix(A, parallel_edges: bool = ..., create_using: Optional[Any] = ...): ...
def to_numpy_recarray(G, nodelist: Optional[Any] = ..., dtype: Optional[Any] = ..., order: Optional[Any] = ...): ...
def to_scipy_sparse_matrix(G, nodelist: Optional[Any] = ..., dtype: Optional[Any] = ..., weight: str = ..., format: str = ...): ...
def from_scipy_sparse_matrix(A, parallel_edges: bool = ..., create_using: Optional[Any] = ..., edge_attribute: str = ...): ...
def to_numpy_array(G, nodelist: Optional[Any] = ..., dtype: Optional[Any] = ..., order: Optional[Any] = ..., multigraph_weight: Any = ..., weight: str = ..., nonedge: float = ...): ...
def from_numpy_array(A, parallel_edges: bool = ..., create_using: Optional[Any] = ...): ...



# Stubs for galaxy.tools.locations.dockstore (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ..locations import ToolLocationResolver

class DockStoreResolver(ToolLocationResolver):
    scheme = ...  # type: str
    def get_tool_source_path(self, uri_like): ...

class _Ga4ghToolClient:
    base_url = ...  # type: Any
    def __init__(self, base_url: str = ...) -> None: ...
    def get_tools(self): ...
    def get_tool(self, tool_id): ...
    def get_tool_version(self, tool_id, version: str = ...): ...
    def get_tool_descriptor(self, tool_id, version: str = ..., tool_type: str = ...): ...
    def get_tool_cwl(self, tool_id, version: str = ..., as_string: bool = ...): ...



# Stubs for networkx.algorithms.connectivity.cuts (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.flow import edmonds_karp
from typing import Any, Optional

default_flow_func = edmonds_karp

def minimum_st_edge_cut(G, s, t, flow_func: Optional[Any] = ..., auxiliary: Optional[Any] = ..., residual: Optional[Any] = ...): ...
def minimum_st_node_cut(G, s, t, flow_func: Optional[Any] = ..., auxiliary: Optional[Any] = ..., residual: Optional[Any] = ...): ...
def minimum_node_cut(G, s: Optional[Any] = ..., t: Optional[Any] = ..., flow_func: Optional[Any] = ...): ...
def minimum_edge_cut(G, s: Optional[Any] = ..., t: Optional[Any] = ..., flow_func: Optional[Any] = ...): ...



# Stubs for networkx.algorithms.centrality.eigenvector (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def eigenvector_centrality(G, max_iter: int = ..., tol: float = ..., nstart: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def eigenvector_centrality_numpy(G, weight: Optional[Any] = ..., max_iter: int = ..., tol: int = ...): ...



# Stubs for galaxy.tools.deps.mulled.mulled_search (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

requests = ...  # type: Any
Schema = ...  # type: Any
TEXT = ...  # type: Any
STORED = ...  # type: Any
create_in = ...  # type: Any
QueryParser = ...  # type: Any
QUAY_API_URL = ...  # type: str

class QuaySearch:
    index = ...  # type: Any
    organization = ...  # type: Any
    def __init__(self, organization) -> None: ...
    def build_index(self): ...
    def search_repository(self, search_string, non_strict): ...
    def get_additional_repository_information(self, repository_string): ...

def main(argv: Optional[Any] = ...): ...



# Stubs for galaxy.tools.deps.dockerfiles (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..deps import commands as commands
from ..deps import docker_util as docker_util
from ..deps.containers import docker_cache_path as docker_cache_path
from ..deps.requirements import parse_requirements_from_xml as parse_requirements_from_xml
from ...tools import loader_directory as loader_directory

log = ...  # type: Any

def docker_host_args(**kwds): ...
def dockerfile_build(path, dockerfile: Optional[Any] = ..., error: Any = ..., **kwds): ...



# Stubs for galaxy.util.checkers (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def check_html(file_path, chunk: Optional[Any] = ...): ...
def check_binary(name, file_path: bool = ...): ...
def check_gzip(file_path): ...
def check_bz2(file_path): ...
def check_zip(file_path): ...
def is_bz2(file_path): ...
def is_gzip(file_path): ...
def check_image(file_path): ...



# Stubs for prov.graph (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

__email__: str
INFERRED_ELEMENT_CLASS: Any

def prov_to_graph(prov_document: Any): ...
def graph_to_prov(g: Any): ...



# Stubs for galaxy.util.aliaspickler (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import pickle
from six.moves import cStringIO as StringIO

class AliasUnpickler(pickle.Unpickler):
    aliases = ...  # type: Any
    def __init__(self, aliases, *args, **kw) -> None: ...
    def find_class(self, module, name): ...

class AliasPickleModule:
    aliases = ...  # type: Any
    def __init__(self, aliases) -> None: ...
    def dump(self, obj, fileobj, protocol: int = ...): ...
    def dumps(self, obj, protocol: int = ...): ...
    def load(self, fileobj): ...
    def loads(self, string): ...



# Stubs for networkx.generators.ego (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def ego_graph(G, n, radius: int = ..., center: bool = ..., undirected: bool = ..., distance: Optional[Any] = ...): ...



# Stubs for galaxy.tools.deps.resolvers.conda (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..resolvers import Dependency, DependencyResolver, InstallableDependencyResolver, ListableDependencyResolver, MappableDependencyResolver, MultipleDependencyResolver, SpecificationPatternDependencyResolver

DEFAULT_ENSURE_CHANNELS = ...  # type: str

class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, ListableDependencyResolver, InstallableDependencyResolver, SpecificationPatternDependencyResolver, MappableDependencyResolver):
    dict_collection_visible_keys = ...  # type: Any
    resolver_type = ...  # type: str
    config_options = ...  # type: Any
    can_uninstall_dependencies = ...  # type: bool
    versionless = ...  # type: Any
    dependency_manager = ...  # type: Any
    conda_prefix_parent = ...  # type: Any
    ensure_channels = ...  # type: Any
    auto_init = ...  # type: Any
    conda_context = ...  # type: Any
    disabled = ...  # type: Any
    auto_install = ...  # type: Any
    copy_dependencies = ...  # type: Any
    def __init__(self, dependency_manager, **kwds) -> None: ...
    def clean(self, **kwds): ...
    def uninstall(self, requirements): ...
    def uninstall_environments(self, environments): ...
    def install_all(self, conda_targets): ...
    def resolve_all(self, requirements, **kwds): ...
    def merged_environment_name(self, conda_targets): ...
    def resolve(self, requirement, **kwds): ...
    def unused_dependency_paths(self, toolbox_requirements_status): ...
    def list_dependencies(self): ...
    def install_dependency(self, name, version, type, **kwds): ...
    @property
    def prefix(self): ...

class MergedCondaDependency(Dependency):
    dict_collection_visible_keys = ...  # type: Any
    dependency_type = ...  # type: str
    activate = ...  # type: Any
    conda_context = ...  # type: Any
    environment_path = ...  # type: Any
    cache_path = ...  # type: Any
    def __init__(self, conda_context, environment_path, exact, name: Optional[Any] = ..., version: Optional[Any] = ..., preserve_python_environment: bool = ...) -> None: ...
    @property
    def exact(self): ...
    @property
    def name(self): ...
    @property
    def version(self): ...
    def shell_commands(self, requirement): ...

class CondaDependency(Dependency):
    dict_collection_visible_keys = ...  # type: Any
    dependency_type = ...  # type: str
    cacheable = ...  # type: bool
    activate = ...  # type: Any
    conda_context = ...  # type: Any
    environment_path = ...  # type: Any
    cache_path = ...  # type: Any
    def __init__(self, conda_context, environment_path, exact, name: Optional[Any] = ..., version: Optional[Any] = ..., preserve_python_environment: bool = ...) -> None: ...
    @property
    def exact(self): ...
    @property
    def name(self): ...
    @property
    def version(self): ...
    def build_cache(self, cache_path): ...
    def set_cache_path(self, cache_path): ...
    def build_environment(self): ...
    def shell_commands(self, requirement): ...



# Stubs for networkx.algorithms.bipartite.matrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def biadjacency_matrix(G, row_order, column_order: Optional[Any] = ..., dtype: Optional[Any] = ..., weight: str = ..., format: str = ...): ...
def from_biadjacency_matrix(A, create_using: Optional[Any] = ..., edge_attribute: str = ...): ...



from typing import Any, Optional
import argparse
class CompletionFinder:
    always_complete_options: Any = ...
    exclude: Any = ...
    validator: Any = ...
    print_suppressed: Any = ...
    completing: bool = ...
    default_completer: Any = ...
    append_space: Any = ...
    def __init__(self, argument_parser: Optional[argparse.ArgumentParser] = ..., always_complete_options: bool = ..., exclude: Optional[Any] = ..., validator: Optional[Any] = ..., print_suppressed: bool = ..., default_completer: Any = ..., append_space: Optional[Any] = ...) -> None: ...
    def __call__(self, argument_parser: argparse.ArgumentParser, always_complete_options: bool = ..., exit_method: Any = ..., output_stream: Optional[Any] = ..., exclude: Optional[Any] = ..., validator: Optional[Any] = ..., print_suppressed: bool = ..., append_space: Optional[Any] = ..., default_completer: Any = ...) -> None: ...
    def collect_completions(self, active_parsers: Any, parsed_args: Any, cword_prefix: Any, debug: Any): ...
    def filter_completions(self, completions: Any): ...
    def quote_completions(self, completions: Any, cword_prequote: Any, last_wordbreak_pos: Any): ...
    def rl_complete(self, text: Any, state: Any): ...
    def get_display_completions(self): ...

class ExclusiveCompletionFinder(CompletionFinder): ...

autocomplete: CompletionFinder




# Stubs for galaxy.jobs.metrics.collectl.subsystems (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class CollectlSubsystem:
    @property
    def command_line_arg(self): ...
    @property
    def name(self, job_directory): ...

class ProcessesSubsystem(CollectlSubsystem):
    command_line_arg = ...  # type: str
    name = ...  # type: str

class CpuSubsystem(CollectlSubsystem):
    command_line_arg = ...  # type: str
    name = ...  # type: str

class DiskSubsystem(CollectlSubsystem):
    command_line_arg = ...  # type: str
    name = ...  # type: str

class NetworkSubsystem(CollectlSubsystem):
    command_line_arg = ...  # type: str
    name = ...  # type: str

class EnvironmentSubsystem(CollectlSubsystem):
    command_line_arg = ...  # type: str
    name = ...  # type: str

class MemorySubsystem(CollectlSubsystem):
    command_line_arg = ...  # type: str
    name = ...  # type: str

def get_subsystem(name): ...



# Stubs for galaxy.tools.deps.resolvers.brewed_tool_shed_packages (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from xml.etree import ElementTree as ET
from .resolver_mixins import UsesHomebrewMixin, UsesInstalledRepositoriesMixin, UsesToolDependencyDirMixin
from ..resolvers import DependencyResolver

class HomebrewToolShedDependencyResolver(DependencyResolver, UsesHomebrewMixin, UsesToolDependencyDirMixin, UsesInstalledRepositoriesMixin):
    resolver_type = ...  # type: str
    def __init__(self, dependency_manager, **kwds) -> None: ...
    def resolve(self, requirement, **kwds): ...

class RawDependencies:
    root = ...  # type: Any
    dependencies = ...  # type: Any
    def __init__(self, dependencies_file) -> None: ...
    def find(self, package_name, package_version): ...

class RawDependency:
    dependencies = ...  # type: Any
    package_el = ...  # type: Any
    repository_el = ...  # type: Any
    def __init__(self, dependencies, package_el, repository_el) -> None: ...
    @property
    def repository_owner(self): ...
    @property
    def repository_name(self): ...
    @property
    def package_name(self): ...
    @property
    def package_version(self): ...



# Stubs for prov (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class Error(Exception): ...

def read(source: Any, format: Optional[Any] = ...): ...

# Names in __all__ with no definition:
#   model



import contextlib
import io
import json
import os
import shutil
import subprocess
from pathlib import Path
from typing import Dict, Generator, List, Mapping, Optional, Tuple, Union

import pytest
from pkg_resources import Requirement, ResolutionError, resource_filename

from cwltool.env_to_stdout import deserialize_env
from cwltool.main import main
from cwltool.singularity import is_version_2_6, is_version_3_or_newer


def force_default_container(default_container_id: str, _: str) -> str:
    return default_container_id


def get_data(filename: str) -> str:
    # normalizing path depending on OS or else it will cause problem when joining path
    filename = os.path.normpath(filename)
    filepath = None
    try:
        filepath = resource_filename(Requirement.parse("cwltool"), filename)
    except ResolutionError:
        pass
    if not filepath or not os.path.isfile(filepath):
        filepath = os.path.join(os.path.dirname(__file__), os.pardir, filename)
    return str(Path(filepath).resolve())


needs_docker = pytest.mark.skipif(
    not bool(shutil.which("docker")),
    reason="Requires the docker executable on the system path.",
)

needs_singularity = pytest.mark.skipif(
    not bool(shutil.which("singularity")),
    reason="Requires the singularity executable on the system path.",
)

needs_singularity_2_6 = pytest.mark.skipif(
    not bool(shutil.which("singularity") and is_version_2_6()),
    reason="Requires that version 2.6.x of singularity executable version is on the system path.",
)

needs_singularity_3_or_newer = pytest.mark.skipif(
    (not bool(shutil.which("singularity"))) or (not is_version_3_or_newer()),
    reason="Requires that version 3.x of singularity executable version is on the system path.",
)

needs_podman = pytest.mark.skipif(
    not bool(shutil.which("podman")),
    reason="Requires the podman executable on the system path.",
)

_env_accepts_null: Optional[bool] = None


def env_accepts_null() -> bool:
    """Return True iff the env command on this host accepts `-0`."""
    global _env_accepts_null
    if _env_accepts_null is None:
        result = subprocess.run(
            ["env", "-0"],
            capture_output=True,
            encoding="utf-8",
        )
        _env_accepts_null = result.returncode == 0

    return _env_accepts_null


def get_main_output(
    args: List[str],
    replacement_env: Optional[Mapping[str, str]] = None,
    extra_env: Optional[Mapping[str, str]] = None,
    monkeypatch: Optional[pytest.MonkeyPatch] = None,
) -> Tuple[Optional[int], str, str]:
    """Run cwltool main.

    args: the command line args to call it with

    replacement_env: a total replacement of the environment

    extra_env: add these to the environment used

    monkeypatch: required if changing the environment

    Returns (return code, stdout, stderr)
    """
    stdout = io.StringIO()
    stderr = io.StringIO()
    if replacement_env is not None:
        assert monkeypatch is not None
        monkeypatch.setattr(os, "environ", replacement_env)

    if extra_env is not None:
        assert monkeypatch is not None
        for k, v in extra_env.items():
            monkeypatch.setenv(k, v)

    try:
        rc = main(argsl=args, stdout=stdout, stderr=stderr)
    except SystemExit as e:
        rc = e.code
    return (
        rc,
        stdout.getvalue(),
        stderr.getvalue(),
    )


def get_tool_env(
    tmp_path: Path,
    flag_args: List[str],
    inputs_file: Optional[str] = None,
    replacement_env: Optional[Mapping[str, str]] = None,
    extra_env: Optional[Mapping[str, str]] = None,
    monkeypatch: Optional[pytest.MonkeyPatch] = None,
    runtime_env_accepts_null: Optional[bool] = None,
) -> Dict[str, str]:
    """Get the env vars for a tool's invocation."""
    # GNU env accepts the -0 option to end each variable's
    # printing with "\0". No such luck on BSD-ish.
    #
    # runtime_env_accepts_null is None => figure it out, otherwise
    # use wrapped bool (because containers).
    if runtime_env_accepts_null is None:
        runtime_env_accepts_null = env_accepts_null()

    args = flag_args.copy()
    if runtime_env_accepts_null:
        args.append(get_data("tests/env3.cwl"))
    else:
        args.append(get_data("tests/env4.cwl"))

    if inputs_file is not None:
        args.append(inputs_file)

    with working_directory(tmp_path):
        rc, stdout, _ = get_main_output(
            args,
            replacement_env=replacement_env,
            extra_env=extra_env,
            monkeypatch=monkeypatch,
        )
        assert rc == 0

        output = json.loads(stdout)
        with open(output["env"]["path"]) as _:
            return deserialize_env(_.read())


@contextlib.contextmanager
def working_directory(path: Union[str, Path]) -> Generator[None, None, None]:
    """Change working directory and returns to previous on exit."""
    prev_cwd = Path.cwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(prev_cwd)



# Stubs for galaxy.util.sleeper (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Sleeper:
    condition = ...  # type: Any
    def __init__(self) -> None: ...
    def sleep(self, seconds): ...
    def wake(self): ...



# Stubs for prov.model (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from prov.constants import *
from prov import Error
from typing import Any, List, Optional

__email__: str
logger: Any

def parse_xsd_datetime(value: Any): ...
def parse_boolean(value: Any): ...

DATATYPE_PARSERS: Any
XSD_DATATYPE_PARSERS: Any

def parse_xsd_types(value: Any, datatype: Any): ...
def first(a_set: Any): ...
def encoding_provn_value(value: Any): ...

class Literal:
    def __init__(self, value: Any, datatype: Optional[Any] = ..., langtag: Optional[Any] = ...) -> None: ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __hash__(self): ...
    @property
    def value(self): ...
    @property
    def datatype(self): ...
    @property
    def langtag(self): ...
    def has_no_langtag(self): ...
    def provn_representation(self): ...

class ProvException(Error): ...
class ProvWarning(Warning): ...

class ProvExceptionInvalidQualifiedName(ProvException):
    qname: Any = ...
    def __init__(self, qname: Any) -> None: ...

class ProvElementIdentifierRequired(ProvException): ...

class ProvRecord:
    FORMAL_ATTRIBUTES: Any = ...
    def __init__(self, bundle: Any, identifier: Any, attributes: Optional[Any] = ...) -> None: ...
    def __hash__(self): ...
    def copy(self): ...
    def get_type(self): ...
    def get_asserted_types(self): ...
    def add_asserted_type(self, type_identifier: Any) -> None: ...
    def get_attribute(self, attr_name: Any): ...
    @property
    def identifier(self): ...
    @property
    def attributes(self): ...
    @property
    def args(self): ...
    @property
    def formal_attributes(self): ...
    @property
    def extra_attributes(self): ...
    @property
    def bundle(self): ...
    @property
    def label(self): ...
    @property
    def value(self): ...
    def add_attributes(self, attributes: Any) -> None: ...
    def __eq__(self, other: Any): ...
    def get_provn(self): ...
    def is_element(self): ...
    def is_relation(self): ...

class ProvElement(ProvRecord):
    def __init__(self, bundle: Any, identifier: Any, attributes: Optional[Any] = ...) -> None: ...
    def is_element(self): ...

class ProvRelation(ProvRecord):
    def is_relation(self): ...

class ProvEntity(ProvElement):
    def wasGeneratedBy(self, activity: Any, time: Optional[Any] = ..., attributes: Optional[Any] = ...): ...
    def wasInvalidatedBy(self, activity: Any, time: Optional[Any] = ..., attributes: Optional[Any] = ...): ...
    def wasDerivedFrom(self, usedEntity: Any, activity: Optional[Any] = ..., generation: Optional[Any] = ..., usage: Optional[Any] = ..., attributes: Optional[Any] = ...): ...
    def wasAttributedTo(self, agent: Any, attributes: Optional[Any] = ...): ...
    def alternateOf(self, alternate2: Any): ...
    def specializationOf(self, generalEntity: Any): ...
    def hadMember(self, entity: Any): ...

class ProvActivity(ProvElement):
    FORMAL_ATTRIBUTES: Any = ...
    def set_time(self, startTime: Optional[Any] = ..., endTime: Optional[Any] = ...) -> None: ...
    def get_startTime(self): ...
    def get_endTime(self): ...
    def used(self, entity: Any, time: Optional[Any] = ..., attributes: Optional[Any] = ...): ...
    def wasInformedBy(self, informant: Any, attributes: Optional[Any] = ...): ...
    def wasStartedBy(self, trigger: Any, starter: Optional[Any] = ..., time: Optional[Any] = ..., attributes: Optional[Any] = ...): ...
    def wasEndedBy(self, trigger: Any, ender: Optional[Any] = ..., time: Optional[Any] = ..., attributes: Optional[Any] = ...): ...
    def wasAssociatedWith(self, agent: Any, plan: Optional[Any] = ..., attributes: Optional[Any] = ...): ...

class ProvGeneration(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvUsage(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvCommunication(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvStart(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvEnd(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvInvalidation(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvDerivation(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvAgent(ProvElement):
    def actedOnBehalfOf(self, responsible: Any, activity: Optional[Any] = ..., attributes: Optional[Any] = ...): ...

class ProvAttribution(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvAssociation(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvDelegation(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvInfluence(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvSpecialization(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvAlternate(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

class ProvMention(ProvSpecialization):
    FORMAL_ATTRIBUTES: Any = ...

class ProvMembership(ProvRelation):
    FORMAL_ATTRIBUTES: Any = ...

PROV_REC_CLS: Any
DEFAULT_NAMESPACES: Any

class NamespaceManager(dict):
    parent: Any = ...
    def __init__(self, namespaces: Optional[Any] = ..., default: Optional[Any] = ..., parent: Optional[Any] = ...) -> None: ...
    def get_namespace(self, uri: Any): ...
    def get_registered_namespaces(self): ...
    def set_default_namespace(self, uri: Any) -> None: ...
    def get_default_namespace(self): ...
    def add_namespace(self, namespace: Any): ...
    def add_namespaces(self, namespaces: Any) -> None: ...
    def valid_qualified_name(self, qname: Any): ...
    def get_anonymous_identifier(self, local_prefix: str = ...): ...

class ProvBundle:
    def __init__(self, records: Optional[Any] = ..., identifier: Optional[Any] = ..., namespaces: Optional[Any] = ..., document: Optional[Any] = ...) -> None: ...
    @property
    def namespaces(self): ...
    @property
    def default_ns_uri(self): ...
    @property
    def document(self): ...
    @property
    def identifier(self): ...
    @property
    def records(self): ...
    def set_default_namespace(self, uri: Any) -> None: ...
    def get_default_namespace(self): ...
    def add_namespace(self, namespace_or_prefix: Any, uri: Optional[Any] = ...): ...
    def get_registered_namespaces(self): ...
    def valid_qualified_name(self, identifier: Any): ...
    def get_records(self, class_or_type_or_tuple: Optional[Any] = ...): ...
    def get_record(self, identifier: Any) -> Optional[List[ProvEntity]]: ...
    def is_document(self): ...
    def is_bundle(self): ...
    def has_bundles(self): ...
    @property
    def bundles(self): ...
    def get_provn(self, _indent_level: int = ...): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    __hash__: Any = ...
    def unified(self): ...
    def update(self, other: Any) -> None: ...
    def new_record(self, record_type: Any, identifier: Any, attributes: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def add_record(self, record: Any): ...
    def entity(self, identifier: Any, other_attributes: Optional[Any] = ...) -> ProvEntity: ...
    def activity(self, identifier: Any, startTime: Optional[Any] = ..., endTime: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def generation(self, entity: Any, activity: Optional[Any] = ..., time: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def usage(self, activity: Any, entity: Optional[Any] = ..., time: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def start(self, activity: Any, trigger: Optional[Any] = ..., starter: Optional[Any] = ..., time: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def end(self, activity: Any, trigger: Optional[Any] = ..., ender: Optional[Any] = ..., time: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def invalidation(self, entity: Any, activity: Optional[Any] = ..., time: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def communication(self, informed: Any, informant: Any, identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def agent(self, identifier: Any, other_attributes: Optional[Any] = ...): ...
    def attribution(self, entity: Any, agent: Any, identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def association(self, activity: Any, agent: Optional[Any] = ..., plan: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def delegation(self, delegate: Any, responsible: Any, activity: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def influence(self, influencee: Any, influencer: Any, identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def derivation(self, generatedEntity: Any, usedEntity: Any, activity: Optional[Any] = ..., generation: Optional[Any] = ..., usage: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def revision(self, generatedEntity: Any, usedEntity: Any, activity: Optional[Any] = ..., generation: Optional[Any] = ..., usage: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def quotation(self, generatedEntity: Any, usedEntity: Any, activity: Optional[Any] = ..., generation: Optional[Any] = ..., usage: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def primary_source(self, generatedEntity: Any, usedEntity: Any, activity: Optional[Any] = ..., generation: Optional[Any] = ..., usage: Optional[Any] = ..., identifier: Optional[Any] = ..., other_attributes: Optional[Any] = ...): ...
    def specialization(self, specificEntity: Any, generalEntity: Any): ...
    def alternate(self, alternate1: Any, alternate2: Any): ...
    def mention(self, specificEntity: Any, generalEntity: Any, bundle: Any): ...
    def collection(self, identifier: Any, other_attributes: Optional[Any] = ...): ...
    def membership(self, collection: Any, entity: Any): ...
    def plot(self, filename: Optional[Any] = ..., show_nary: bool = ..., use_labels: bool = ..., show_element_attributes: bool = ..., show_relation_attributes: bool = ...) -> None: ...
    wasGeneratedBy: Any = ...
    used: Any = ...
    wasStartedBy: Any = ...
    wasEndedBy: Any = ...
    wasInvalidatedBy: Any = ...
    wasInformedBy: Any = ...
    wasAttributedTo: Any = ...
    wasAssociatedWith: Any = ...
    actedOnBehalfOf: Any = ...
    wasInfluencedBy: Any = ...
    wasDerivedFrom: Any = ...
    wasRevisionOf: Any = ...
    wasQuotedFrom: Any = ...
    hadPrimarySource: Any = ...
    alternateOf: Any = ...
    specializationOf: Any = ...
    mentionOf: Any = ...
    hadMember: Any = ...

class ProvDocument(ProvBundle):
    def __init__(self, records: Optional[Any] = ..., namespaces: Optional[Any] = ...) -> None: ...
    def __eq__(self, other: Any): ...
    def is_document(self): ...
    def is_bundle(self): ...
    def has_bundles(self): ...
    @property
    def bundles(self): ...
    def flattened(self) -> ProvDocument: ...
    def unified(self): ...
    def update(self, other: Any) -> None: ...
    def add_bundle(self, bundle: Any, identifier: Optional[Any] = ...) -> None: ...
    def bundle(self, identifier: Any): ...
    def serialize(self, destination: Optional[Any] = ..., format: str = ..., **args: Any): ...
    @staticmethod
    def deserialize(source: Optional[Any] = ..., content: Optional[Any] = ..., format: str = ..., **args: Any): ...

def sorted_attributes(element: Any, attributes: Any): ...



# Stubs for coloredlogs (Python 3.7)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import logging
from typing import Any, Optional

WINDOWS: Any
NEED_COLORAMA = WINDOWS
DEFAULT_LOG_LEVEL: Any
DEFAULT_LOG_FORMAT: str
DEFAULT_DATE_FORMAT: str
CHROOT_FILES: Any
CAN_USE_BOLD_FONT: Any
DEFAULT_FIELD_STYLES: Any
DEFAULT_LEVEL_STYLES: Any
DEFAULT_FORMAT_STYLE: str
FORMAT_STYLE_PATTERNS: Any

def auto_install() -> None: ...
def install(level: Optional[Any] = ..., **kw: Any): ...
def check_style(value: Any): ...
def increase_verbosity() -> None: ...
def decrease_verbosity() -> None: ...
def is_verbose(): ...
def get_level(): ...
def set_level(level: Any) -> None: ...
def adjust_level(logger: Any, level: Any) -> None: ...
def find_defined_levels(): ...
def level_to_number(value: Any): ...
def find_level_aliases(): ...
def parse_encoded_styles(text: Any, normalize_key: Optional[Any] = ...): ...
def find_hostname(use_chroot: bool = ...): ...
def find_program_name(): ...
def replace_handler(logger: Any, match_handler: Any, reconfigure: Any): ...
def find_handler(logger: Any, match_handler: Any): ...
def match_stream_handler(handler: Any, streams: Any = ...): ...
def walk_propagation_tree(logger: Any) -> None: ...

class BasicFormatter(logging.Formatter):
    def formatTime(self, record: Any, datefmt: Optional[Any] = ...): ...

class ColoredFormatter(BasicFormatter):
    nn: Any = ...
    log_record_factory: Any = ...
    level_styles: Any = ...
    field_styles: Any = ...
    def __init__(self, fmt: Optional[Any] = ..., datefmt: Optional[Any] = ..., level_styles: Optional[Any] = ..., field_styles: Optional[Any] = ..., style: Any = ...) -> None: ...
    def colorize_format(self, fmt: Any, style: Any = ...): ...
    def format(self, record: Any): ...

class Empty: ...

class HostNameFilter(logging.Filter):
    @classmethod
    def install(cls, handler: Any, fmt: Optional[Any] = ..., use_chroot: bool = ..., style: Any = ...): ...
    hostname: Any = ...
    def __init__(self, use_chroot: bool = ...) -> None: ...
    def filter(self, record: Any): ...

class ProgramNameFilter(logging.Filter):
    @classmethod
    def install(cls, handler: Any, fmt: Any, programname: Optional[Any] = ..., style: Any = ...): ...
    programname: Any = ...
    def __init__(self, programname: Optional[Any] = ...) -> None: ...
    def filter(self, record: Any): ...

class StandardErrorHandler(logging.StreamHandler):
    def __init__(self, level: Any = ...) -> None: ...
    @property
    def stream(self): ...

class FormatStringParser:
    style: Any = ...
    capturing_pattern: Any = ...
    raw_pattern: Any = ...
    tokenize_pattern: Any = ...
    name_pattern: Any = ...
    def __init__(self, style: Any = ...) -> None: ...
    def contains_field(self, format_string: Any, field_name: Any): ...
    def get_field_names(self, format_string: Any): ...
    def get_grouped_pairs(self, format_string: Any): ...
    def get_pairs(self, format_string: Any) -> None: ...
    def get_pattern(self, field_name: Any): ...
    def get_tokens(self, format_string: Any): ...

class FormatStringToken: ...

class NameNormalizer:
    aliases: Any = ...
    def __init__(self) -> None: ...
    def normalize_name(self, name: Any): ...
    def normalize_keys(self, value: Any): ...
    def get(self, normalized_dict: Any, name: Any): ...



# Stubs for galaxy.util.simplegraph (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class SimpleGraphNode:
    index = ...  # type: Any
    data = ...  # type: Any
    def __init__(self, index, **data) -> None: ...

class SimpleGraphEdge:
    source_index = ...  # type: Any
    target_index = ...  # type: Any
    data = ...  # type: Any
    def __init__(self, source_index, target_index, **data) -> None: ...

class SimpleGraph:
    nodes = ...  # type: Any
    edges = ...  # type: Any
    def __init__(self, nodes: Optional[Any] = ..., edges: Optional[Any] = ...) -> None: ...
    def add_node(self, node_id, **data): ...
    def add_edge(self, source_id, target_id, **data): ...
    def gen_node_dicts(self): ...
    def gen_edge_dicts(self): ...
    def as_dict(self): ...



# Stubs for networkx.algorithms.similarity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from operator import *
from typing import Any, Optional

def graph_edit_distance(G1, G2, node_match: Optional[Any] = ..., edge_match: Optional[Any] = ..., node_subst_cost: Optional[Any] = ..., node_del_cost: Optional[Any] = ..., node_ins_cost: Optional[Any] = ..., edge_subst_cost: Optional[Any] = ..., edge_del_cost: Optional[Any] = ..., edge_ins_cost: Optional[Any] = ..., upper_bound: Optional[Any] = ...): ...
def optimal_edit_paths(G1, G2, node_match: Optional[Any] = ..., edge_match: Optional[Any] = ..., node_subst_cost: Optional[Any] = ..., node_del_cost: Optional[Any] = ..., node_ins_cost: Optional[Any] = ..., edge_subst_cost: Optional[Any] = ..., edge_del_cost: Optional[Any] = ..., edge_ins_cost: Optional[Any] = ..., upper_bound: Optional[Any] = ...): ...
def optimize_graph_edit_distance(G1, G2, node_match: Optional[Any] = ..., edge_match: Optional[Any] = ..., node_subst_cost: Optional[Any] = ..., node_del_cost: Optional[Any] = ..., node_ins_cost: Optional[Any] = ..., edge_subst_cost: Optional[Any] = ..., edge_del_cost: Optional[Any] = ..., edge_ins_cost: Optional[Any] = ..., upper_bound: Optional[Any] = ...): ...
def optimize_edit_paths(G1, G2, node_match: Optional[Any] = ..., edge_match: Optional[Any] = ..., node_subst_cost: Optional[Any] = ..., node_del_cost: Optional[Any] = ..., node_ins_cost: Optional[Any] = ..., edge_subst_cost: Optional[Any] = ..., edge_del_cost: Optional[Any] = ..., edge_ins_cost: Optional[Any] = ..., upper_bound: Optional[Any] = ..., strictly_decreasing: bool = ...): ...



# Stubs for galaxy.tools.deps (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional, List
from .requirements import ToolRequirement as ToolRequirement, ToolRequirements as ToolRequirements
from .resolvers import NullDependency as NullDependency
from .resolvers.conda import CondaDependencyResolver as CondaDependencyResolver
from .resolvers.galaxy_packages import GalaxyPackageDependencyResolver as GalaxyPackageDependencyResolver
from .resolvers.tool_shed_packages import ToolShedPackageDependencyResolver as ToolShedPackageDependencyResolver

log = ...  # type: Any
CONFIG_VAL_NOT_FOUND = ...  # type: Any

def build_dependency_manager(app_config_dict: Any, resolution_config_dict: Any, conf_file: Any) -> DependencyManager: ...

class NullDependencyManager:
    dependency_resolvers = ...  # type: Any
    def uses_tool_shed_dependencies(self): ...
    def dependency_shell_commands(self, requirements: ToolRequirements, **kwds) -> List[str]: ...
    def find_dep(self, name, version: Optional[Any] = ..., type: str = ..., **kwds): ...

class DependencyManager:
    default_base_path = ...  # type: Any
    resolver_classes = ...  # type: Any
    dependency_resolvers = ...  # type: Any
    def __init__(self, default_base_path, conf_file: Optional[Any] = ..., app_config: Any = ...) -> None: ...
    def get_resolver_option(self, resolver, key, explicit_resolver_options: Any = ...): ...
    def get_app_option(self, key, default: Optional[Any] = ...): ...
    def dependency_shell_commands(self, requirements: ToolRequirements, **kwds) -> List[str]: ...
    def requirements_to_dependencies(self, requirements, **kwds): ...
    def uses_tool_shed_dependencies(self): ...
    def find_dep(self, name, version: Optional[Any] = ..., type: str = ..., **kwds): ...

class CachedDependencyManager(DependencyManager):
    tool_dependency_cache_dir = ...  # type: Any
    def __init__(self, default_base_path, conf_file: Optional[Any] = ..., app_config: Any = ..., tool_dependency_cache_dir: Optional[Any] = ...) -> None: ...
    def build_cache(self, requirements, **kwds): ...
    def dependency_shell_commands(self, requirements: ToolRequirements, **kwds) -> List[str]: ...
    def hash_dependencies(self, resolved_dependencies): ...
    def get_hashed_dependencies_path(self, resolved_dependencies): ...



# Stubs for prov.serializers.provrdf (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from prov.serializers import Error, Serializer
from typing import Any, Optional

__email__: str

class ProvRDFException(Error): ...

class AnonymousIDGenerator:
    def __init__(self) -> None: ...
    def get_anon_id(self, obj: Any, local_prefix: str = ...): ...

LITERAL_XSDTYPE_MAP: Any

def attr2rdf(attr: Any): ...
def valid_qualified_name(bundle: Any, value: Any, xsd_qname: bool = ...): ...

class ProvRDFSerializer(Serializer):
    def serialize(self, stream: Optional[Any] = ..., rdf_format: str = ..., **kwargs: Any) -> None: ...
    document: Any = ...
    def deserialize(self, stream: Any, rdf_format: str = ..., **kwargs: Any): ...
    def valid_identifier(self, value: Any): ...
    def encode_rdf_representation(self, value: Any): ...
    def decode_rdf_representation(self, literal: Any, graph: Any): ...
    def encode_document(self, document: Any): ...
    def encode_container(self, bundle: Any, container: Optional[Any] = ..., identifier: Optional[Any] = ...): ...
    def decode_document(self, content: Any, document: Any) -> None: ...
    def decode_container(self, graph: Any, bundle: Any) -> None: ...

def walk(children: Any, level: int = ..., path: Optional[Any] = ..., usename: bool = ...): ...
def literal_rdf_representation(literal: Any): ...



# Stubs for networkx.algorithms.bipartite (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.bipartite.basic import *
from networkx.algorithms.bipartite.centrality import *
from networkx.algorithms.bipartite.cluster import *
from networkx.algorithms.bipartite.covering import *
from networkx.algorithms.bipartite.edgelist import *
from networkx.algorithms.bipartite.matching import *
from networkx.algorithms.bipartite.matrix import *
from networkx.algorithms.bipartite.projection import *
from networkx.algorithms.bipartite.redundancy import *
from networkx.algorithms.bipartite.spectral import *
from networkx.algorithms.bipartite.generators import *



# Stubs for galaxy.tools.deps.resolvers.homebrew (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .resolver_mixins import UsesHomebrewMixin
from ..resolvers import DependencyResolver

class HomebrewDependencyResolver(DependencyResolver, UsesHomebrewMixin):
    resolver_type = ...  # type: str
    versionless = ...  # type: Any
    prefer_version = ...  # type: Any
    def __init__(self, dependency_manager, **kwds) -> None: ...
    def resolve(self, requirement, **kwds): ...



# Stubs for networkx.readwrite.adjlist (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def generate_adjlist(G, delimiter: str = ...): ...
def write_adjlist(G, path, comments: str = ..., delimiter: str = ..., encoding: str = ...): ...
def parse_adjlist(lines, comments: str = ..., delimiter: Optional[Any] = ..., create_using: Optional[Any] = ..., nodetype: Optional[Any] = ...): ...
def read_adjlist(path, comments: str = ..., delimiter: Optional[Any] = ..., create_using: Optional[Any] = ..., nodetype: Optional[Any] = ..., encoding: str = ...): ...



# Stubs for galaxy.util.dictifiable (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class Dictifiable:
    def to_dict(self, view: str = ..., value_mapper: Optional[Any] = ...): ...



# Stubs for galaxy.tools.cwl.cwltool_deps (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from cwltool import main as main, workflow as workflow, process as process, pathmapper as pathmapper
from cwltool import load_tool as load_tool
import shellescape as shellescape
import schema_salad as schema_salad
from schema_salad import ref_resolver as ref_resolver

main = ...  # type: Any
workflow = ...  # type: Any
process = ...  # type: Any
pathmapper = ...  # type: Any
load_tool = ...  # type: Any
shellescape = ...  # type: Any
schema_salad = ...  # type: Any
ref_resolver = ...  # type: Any
needs_shell_quoting = ...  # type: Any

def ensure_cwltool_available(): ...



# Stubs for prov.constants (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

__email__: str
XSD: Any
PROV: Any
XSI: Any
PROV_ENTITY: Any
PROV_ACTIVITY: Any
PROV_GENERATION: Any
PROV_USAGE: Any
PROV_COMMUNICATION: Any
PROV_START: Any
PROV_END: Any
PROV_INVALIDATION: Any
PROV_DERIVATION: Any
PROV_AGENT: Any
PROV_ATTRIBUTION: Any
PROV_ASSOCIATION: Any
PROV_DELEGATION: Any
PROV_INFLUENCE: Any
PROV_BUNDLE: Any
PROV_ALTERNATE: Any
PROV_SPECIALIZATION: Any
PROV_MENTION: Any
PROV_MEMBERSHIP: Any
PROV_N_MAP: Any
ADDITIONAL_N_MAP: Any
PROV_BASE_CLS: Any
PROV_ATTR_ENTITY: Any
PROV_ATTR_ACTIVITY: Any
PROV_ATTR_TRIGGER: Any
PROV_ATTR_INFORMED: Any
PROV_ATTR_INFORMANT: Any
PROV_ATTR_STARTER: Any
PROV_ATTR_ENDER: Any
PROV_ATTR_AGENT: Any
PROV_ATTR_PLAN: Any
PROV_ATTR_DELEGATE: Any
PROV_ATTR_RESPONSIBLE: Any
PROV_ATTR_GENERATED_ENTITY: Any
PROV_ATTR_USED_ENTITY: Any
PROV_ATTR_GENERATION: Any
PROV_ATTR_USAGE: Any
PROV_ATTR_SPECIFIC_ENTITY: Any
PROV_ATTR_GENERAL_ENTITY: Any
PROV_ATTR_ALTERNATE1: Any
PROV_ATTR_ALTERNATE2: Any
PROV_ATTR_BUNDLE: Any
PROV_ATTR_INFLUENCEE: Any
PROV_ATTR_INFLUENCER: Any
PROV_ATTR_COLLECTION: Any
PROV_ATTR_TIME: Any
PROV_ATTR_STARTTIME: Any
PROV_ATTR_ENDTIME: Any
PROV_ATTRIBUTE_QNAMES: Any
PROV_ATTRIBUTE_LITERALS: Any
PROV_ATTRIBUTES: Any
PROV_RECORD_ATTRIBUTES: Any
PROV_RECORD_IDS_MAP: Any
PROV_ID_ATTRIBUTES_MAP: Any
PROV_ATTRIBUTES_ID_MAP: Any
PROV_TYPE: Any
PROV_LABEL: Any
PROV_VALUE: Any
PROV_LOCATION: Any
PROV_ROLE: Any
PROV_QUALIFIEDNAME: Any
XSD_ANYURI: Any
XSD_QNAME: Any
XSD_DATETIME: Any
XSD_TIME: Any
XSD_DATE: Any
XSD_STRING: Any
XSD_BOOLEAN: Any
XSD_INTEGER: Any
XSD_LONG: Any
XSD_INT: Any
XSD_SHORT: Any
XSD_BYTE: Any
XSD_NONNEGATIVEINTEGER: Any
XSD_UNSIGNEDLONG: Any
XSD_UNSIGNEDINT: Any
XSD_UNSIGNEDSHORT: Any
XSD_UNSIGNEDBYTE: Any
XSD_POSITIVEINTEGER: Any
XSD_NONPOSITIVEINTEGER: Any
XSD_NEGATIVEINTEGER: Any
XSD_FLOAT: Any
XSD_DOUBLE: Any
XSD_DECIMAL: Any



# Stubs for galaxy.tools.fetcher (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class ToolLocationFetcher:
    resolver_classes = ...  # type: Any
    def __init__(self) -> None: ...
    def to_tool_path(self, path_or_uri_like, **kwds): ...



# Stubs for galaxy.jobs.metrics.collectl.stats (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class StatisticsTracker:
    min = ...  # type: Any
    max = ...  # type: Any
    count = ...  # type: int
    sum = ...  # type: int
    def __init__(self) -> None: ...
    def track(self, value): ...
    @property
    def avg(self): ...



# Stubs for galaxy.tools.parser.util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .interface import ToolStdioExitCode as ToolStdioExitCode
from .interface import ToolStdioRegex as ToolStdioRegex

def error_on_exit_code(): ...
def aggressive_error_checks(): ...



# Stubs for galaxy.tools.deps.commands (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from galaxy.util import which as which

def redirecting_io(sys: Any = ...): ...
def redirect_aware_commmunicate(p, sys: Any = ...): ...
def shell(cmds, env: Optional[Any] = ..., **kwds): ...
def shell_process(cmds, env: Optional[Any] = ..., **kwds): ...
def execute(cmds): ...
def argv_to_str(command_argv, quote: bool = ...): ...
def download_command(url, to: Any = ..., quote_url: bool = ...): ...

class CommandLineException(Exception):
    command = ...  # type: Any
    stdout = ...  # type: Any
    stderr = ...  # type: Any
    returncode = ...  # type: Any
    message = ...  # type: Any
    def __init__(self, command, stdout, stderr, returncode) -> None: ...



# Stubs for galaxy.objectstore.rods (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from posixpath import basename as path_basename
from posixpath import dirname as path_dirname
from posixpath import join as path_join
from ..objectstore import DiskObjectStore as DiskObjectStore, local_extra_dirs as local_extra_dirs

irods = ...  # type: Any
IRODS_IMPORT_MESSAGE = ...  # type: str
log = ...  # type: Any

class IRODSObjectStore(DiskObjectStore):
    cache_path = ...  # type: Any
    default_resource = ...  # type: Any
    root_collection_path = ...  # type: Any
    root_collection = ...  # type: Any
    def __init__(self, config, file_path: Optional[Any] = ..., extra_dirs: Optional[Any] = ...) -> None: ...
    def exists(self, obj, **kwargs): ...
    def create(self, obj, **kwargs): ...
    def empty(self, obj, **kwargs): ...
    def size(self, obj, **kwargs): ...
    def delete(self, obj, entire_dir: bool = ..., **kwargs): ...
    def get_data(self, obj, start: int = ..., count: int = ..., **kwargs): ...
    def get_filename(self, obj, **kwargs): ...
    def update_from_file(self, obj, file_name: Optional[Any] = ..., create: bool = ..., **kwargs): ...
    def get_object_url(self, obj, **kwargs): ...
    def get_store_usage_percent(self): ...

def rods_connect(): ...



# Stubs for galaxy.util.filelock (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class FileLockException(Exception): ...

class FileLock:
    is_locked = ...  # type: bool
    lockfile = ...  # type: Any
    file_name = ...  # type: Any
    timeout = ...  # type: Any
    delay = ...  # type: Any
    def __init__(self, file_name, timeout: int = ..., delay: float = ...) -> None: ...
    fd = ...  # type: Any
    def acquire(self): ...
    def release(self): ...
    def __enter__(self): ...
    def __exit__(self, type, value, traceback): ...
    def __del__(self): ...



# Stubs for galaxy.tools.cwl (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .cwltool_deps import needs_shell_quoting as needs_shell_quoting, shellescape as shellescape
from .parser import tool_proxy as tool_proxy, workflow_proxy as workflow_proxy
from .representation import to_cwl_job as to_cwl_job, to_galaxy_parameters as to_galaxy_parameters
from .runtime_actions import handle_outputs as handle_outputs



# Stubs for networkx.algorithms.assortativity.pairs (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def node_attribute_xy(G, attribute, nodes: Optional[Any] = ...): ...
def node_degree_xy(G, x: str = ..., y: str = ..., weight: Optional[Any] = ..., nodes: Optional[Any] = ...): ...



# Stubs for networkx.drawing.nx_pylab (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def draw(G, pos: Optional[Any] = ..., ax: Optional[Any] = ..., **kwds): ...
def draw_networkx(G, pos: Optional[Any] = ..., arrows: bool = ..., with_labels: bool = ..., **kwds): ...
def draw_networkx_nodes(G, pos, nodelist: Optional[Any] = ..., node_size: int = ..., node_color: str = ..., node_shape: str = ..., alpha: float = ..., cmap: Optional[Any] = ..., vmin: Optional[Any] = ..., vmax: Optional[Any] = ..., ax: Optional[Any] = ..., linewidths: Optional[Any] = ..., edgecolors: Optional[Any] = ..., label: Optional[Any] = ..., **kwds): ...
def draw_networkx_edges(G, pos, edgelist: Optional[Any] = ..., width: float = ..., edge_color: str = ..., style: str = ..., alpha: float = ..., arrowstyle: str = ..., arrowsize: int = ..., edge_cmap: Optional[Any] = ..., edge_vmin: Optional[Any] = ..., edge_vmax: Optional[Any] = ..., ax: Optional[Any] = ..., arrows: bool = ..., label: Optional[Any] = ..., node_size: int = ..., nodelist: Optional[Any] = ..., node_shape: str = ..., **kwds): ...
def draw_networkx_labels(G, pos, labels: Optional[Any] = ..., font_size: int = ..., font_color: str = ..., font_family: str = ..., font_weight: str = ..., alpha: float = ..., bbox: Optional[Any] = ..., ax: Optional[Any] = ..., **kwds): ...
def draw_networkx_edge_labels(G, pos, edge_labels: Optional[Any] = ..., label_pos: float = ..., font_size: int = ..., font_color: str = ..., font_family: str = ..., font_weight: str = ..., alpha: float = ..., bbox: Optional[Any] = ..., ax: Optional[Any] = ..., rotate: bool = ..., **kwds): ...
def draw_circular(G, **kwargs): ...
def draw_kamada_kawai(G, **kwargs): ...
def draw_random(G, **kwargs): ...
def draw_spectral(G, **kwargs): ...
def draw_spring(G, **kwargs): ...
def draw_shell(G, **kwargs): ...



# Stubs for galaxy.tools.deps.container_resolvers.explicit (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from ..container_resolvers import ContainerResolver

class ExplicitContainerResolver(ContainerResolver):
    resolver_type = ...  # type: str
    def resolve(self, enabled_container_types, tool_info): ...



# Stubs for galaxy.tools.verify.asserts.xml (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def to_xml(output): ...
def xml_find_text(output, path): ...
def xml_find(output, path): ...
def assert_is_valid_xml(output): ...
def assert_has_element_with_path(output, path): ...
def assert_has_n_elements_with_path(output, path, n): ...
def assert_element_text_matches(output, path, expression): ...
def assert_element_text_is(output, path, text): ...
def assert_attribute_matches(output, path, attribute, expression): ...
def assert_attribute_is(output, path, attribute, text): ...
def assert_element_text(output, path, verify_assertions_function, children): ...
